# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#          Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Qt UI."""

import os
import Queue
import shutil
import tempfile

from PyQt4 import QtCore

from ubuntuone.credentials import (
    APP_NAME,
    TC_URL,
    PING_URL,
)

import ubuntu_sso

from ubuntuone_installer.gui.qt import gui
from ubuntuone_installer.gui.qt.tests import BaseTestCase
from ubuntuone_installer.gui.qt.embedded_sso import UbuntuSSOClientGUI
from ubuntuone_installer.gui.qt import local_folders
from ubuntuone_installer.gui.qt import setup_account
from ubuntuone_installer.gui.qt.ui import (
    setup_account_ui
)


TOKEN = {u'consumer_key': u'xQ7xDAz',
         u'consumer_secret': u'KzCJWCTNbbntwfyCKKjomJDzlgqxLy',
         u'token_name': u'test',
         u'token': u'GkInOfSMGwTXAUoVQwLUoPxElEEUdhsLVNTPhxHJDUIeHCPNEo',
         u'token_secret': u'qFYImEtlczPbsCnYyuwLoPDlPEnvNcIktZphPQklAWrvyfFMV'}


class FakeCredentials(object):
    """A very dummy Credentials object."""

    def __init__(self, *a, **kw):
        self.args = a
        self.kwargs = kw


class FakeController(object):
    """A fake controller for the tests."""

    def __init__(self, *args, **kwargs):
        self.args = (args, kwargs)

    # pylint: disable=C0103
    def setupUi(self, view):
        """Fake the setup."""
    # pylint: enable=C0103
    is_fake = True


class MainWindowTestCase(BaseTestCase):
    """Test the qt main window."""

    class_ui = gui.MainWindow

    def setUp(self):
        """Initialize this test instance."""
        # Faking each SSO object instead of doing it lower
        # so we don't rely on any SSO behaviour
        self.patch(gui, "Credentials", FakeCredentials)
        self.patch(gui, "SetUpAccountController", FakeController)
        self.patch(gui, "TosController", FakeController)
        self.patch(gui, "EmailVerificationController", FakeController)
        self.patch(gui, "CurrentUserController", FakeController)
        self.patch(gui, "SuccessController", FakeController)
        self.patch(gui, "ForgottenPasswordController", FakeController)
        self.patch(gui, "ResetPasswordController", FakeController)
        self.patch(gui, "ResetPasswordController", FakeController)
        super(MainWindowTestCase, self).setUp()

    def test_done_calls_custom_close_callback(self):
        """When closing the window, close_callback is called."""
        self.ui.close_callback = self._set_called
        self.ui.done(result=0)
        self.assertEqual(self._called,
                         ((), {}), 'close_callback called.')

    def test_close_callback_can_be_none(self):
        """The close_callback can be None."""
        self.ui.close_callback = None
        self.ui.done(result=0)
        # world did not explode

    def test_start_control_panel_on_finishing(self):
        """If done is called with result=1, the control panel
        should be called.
        """
        self.patch(gui.subprocess, "Popen", self._set_called)
        self.ui.done(result=1)
        self.assertEqual(self._called,
            ((['ubuntuone-control-panel-qt'],), {}))

    def test_not_start_control_panel_on_cancel(self):
        """If done is called with result=0, the control panel
        should NOT be called.
        """
        self.patch(gui.subprocess, "Popen", self._set_called)
        self.ui.done(result=0)
        self.assertEqual(self._called, False)

    def test_sync_now_shows_message(self):
        """If the user selects 'Sync Now' the congrats page shows a message"""
        congrats_page = self.ui.page(self.ui.CONGRATULATIONS_PAGE)
        congrats_page.setField("sync_now", True)

        # Show the congrats page
        self.ui.setStartId(self.ui.CONGRATULATIONS_PAGE)
        self.ui.restart()
        self.ui.show()

        # Check the properties have the right values
        self.assertEqual(congrats_page.field("sync_later").toBool(),
            False)
        self.assertEqual(
            congrats_page.ui.progressContainer.isVisible(), True)

    def test_sync_later_hides_message(self):
        """If the user selects 'Sync Later' the congrats page
        hides a message"""
        congrats_page = self.ui.page(self.ui.CONGRATULATIONS_PAGE)
        congrats_page.setField("sync_later", True)

        # Show the congrats page
        self.ui.setStartId(self.ui.CONGRATULATIONS_PAGE)
        self.ui.restart()
        self.ui.show()

        # Check the properties have the right values
        self.assertEqual(congrats_page.field("sync_now").toBool(),
            False)
        self.assertEqual(
            congrats_page.ui.progressContainer.isVisible(), False)

    def test_credential_parameters(self):
        """Compare credential parameters with control panel's."""
        self.assertEqual(self.ui.creds.kwargs, {
            'app_name': APP_NAME,
            'ui_module': "ubuntuone_installer.gui.qt.embedded_sso",
            'tc_url': TC_URL,
            'ping_url': PING_URL,
        })

    def test_current_user_controller_parameters(self):
        """Compare controller parameters with expected values."""
        self.assertEqual(self.ui.current_user_controller.args,
        ((), {'title': gui.SIGN_IN,
              'subtitle': gui.SIGN_IN_SUBTITLE}))


class SSOGuiTestCase(BaseTestCase):
    """Test the custom SSO GUI."""

    class_ui = UbuntuSSOClientGUI

    def setUp(self):
        """Initialize this test instance."""
        self.kwargs = {
            'app_name': APP_NAME,
            'tc_url': TC_URL,
            'help_text': '',
        }
        QtCore.QCoreApplication.instance().window = True
        super(SSOGuiTestCase, self).setUp()

    def test_sso_client_gui(self):
        """Ensure the class stores the right parameters."""
        self.assertEqual(isinstance(
            self.ui.controller,
            ubuntu_sso.qt.controllers.UbuntuSSOWizardController), True)
        self.assertEqual(self.ui.view, True)


class LocalFoldersTestCase(BaseTestCase):
    """Test the LocalFoldersPage code."""

    class_ui = local_folders.LocalFoldersPage

    def setUp(self):
        """Initialize this test instance."""
        # Create a test folder with a known size
        self.tmpdir = tempfile.mkdtemp()
        f = open(os.path.join(self.tmpdir, 'test_file'), 'wb')
        f.write(" " * 600)
        f.close()
        os.mkdir(os.path.join(self.tmpdir, 'test_dir'))
        f = open(os.path.join(self.tmpdir, 'test_dir', 'test_file_2'), 'wb')
        f.write(" " * 737)
        f.close()
        super(LocalFoldersTestCase, self).setUp()

    def tearDown(self):
        """Remove the temporary tree."""
        shutil.rmtree(self.tmpdir)
        BaseTestCase.tearDown(self)

    def test_size_calculation(self):
        """Test the recursive folder size calculation."""
        self.queue = Queue.Queue()
        self.csize = local_folders.CalculateSize(self.tmpdir, self.queue)
        self.csize.run()
        path, size = self.queue.get()
        self.assertEqual(path, self.tmpdir)
        self.assertEqual(size, 1337)

    def test_item_addition_removal(self):
        """Add an item (plus the default one), then remove them."""
        self.ui.add_folder(self.tmpdir)
        self.assertEqual(2, self.ui.ui.folder_list.topLevelItemCount())
        self.ui.on_folder_list_itemClicked(
            self.ui.ui.folder_list.topLevelItem(0), 2)
        self.ui.on_folder_list_itemClicked(
            self.ui.ui.folder_list.topLevelItem(0), 2)
        self.assertEqual(0, self.ui.ui.folder_list.topLevelItemCount())
        self.assertEqual({}, self.ui.items)

    def test_total_size(self):
        """Test that the header reflects the change in item sizes."""
        while self.ui.ui.folder_list.topLevelItemCount():
            self.ui.on_folder_list_itemClicked(
                self.ui.ui.folder_list.topLevelItem(0), 2)
        self.assertEqual(0, self.ui.ui.folder_list.topLevelItemCount())
        item = self.ui.add_folder(self.tmpdir)
        item.size = 1337
        self.ui.update_sizes()
        self.assertEqual(unicode(self.ui.ui.folder_list.headerItem().text(1)),
            u"Space (1337)")

    def test_add_twice(self):
        """Behaviour for adding the same folder twice:

        * It's added only once.
        """
        while self.ui.ui.folder_list.topLevelItemCount():
            self.ui.on_folder_list_itemClicked(
                self.ui.ui.folder_list.topLevelItem(0), 2)
        self.assertEqual(0, self.ui.ui.folder_list.topLevelItemCount())
        self.ui.add_folder(self.tmpdir)
        self.ui.add_folder(self.tmpdir)
        self.assertEqual(1, self.ui.ui.folder_list.topLevelItemCount())

    def test_add_missing_folder(self):
        """Behaviour for adding a folder that doesn't exist:

        * It's added.
        * Has size 0.
        """

        while self.ui.ui.folder_list.topLevelItemCount():
            self.ui.on_folder_list_itemClicked(
                self.ui.ui.folder_list.topLevelItem(0), 2)
        self.assertEqual(0, self.ui.ui.folder_list.topLevelItemCount())
        item = self.ui.add_folder(os.path.join("xyzzy", "xyzzy", "xyzzy"))
        self.assertEqual(1, self.ui.ui.folder_list.topLevelItemCount())
        item.thread.run()
        self.ui.update_sizes()
        self.assertEqual(0, item.size)
        # world did not explode

    def test_over_quota(self):
        """After removing all folders, offer_frame should be hidden.

        Push the user over quota, it should be visible.
        """
        self.patch(self.ui.ui.offer_frame, "setVisible", self._set_called)
        while self.ui.ui.folder_list.topLevelItemCount():
            self.ui.on_folder_list_itemClicked(
                self.ui.ui.folder_list.topLevelItem(0), 2)
        self.assertEqual(0, self.ui.ui.folder_list.topLevelItemCount())
        self.ui.update_sizes()
        self.assertEqual(self._called, ((False,), {}))
        self.ui.show_hide_offer(self.ui.quota() + 1)
        self.assertEqual(self._called, ((True,), {}))


class SetupAccountTestCase(BaseTestCase):
    """Test the SetupAccountPage code."""

    class_ui = setup_account.SetupAccountPage
    kwargs = dict(
        ui=setup_account_ui.Ui_SetUpAccountPage(),
        controller=FakeController(),
        parent=None,
    )

    def setUp(self):
        """Initialize this test instance."""
        super(SetupAccountTestCase, self).setUp()

    def test_empty_name(self):
        """Status when the name field is empty:

        * Name Assistance label visible
        * Correct error message in it
        """
        self.ui.ui.name_edit.setText("")
        self.ui.name_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.name_assistance.isVisible(), True)
        self.assertEqual(
            unicode(self.ui.ui.name_assistance.text()),
            setup_account.bad % setup_account.EMPTY_NAME
        )
        self.ui.hide()

    def test_blank_name(self):
        """Status when the name field is blank (spaces):

        * Name Assistance label visible
        * Correct error message in it
        """
        self.ui.ui.name_edit.setText("  ")
        self.ui.name_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.name_assistance.isVisible(), True)
        self.assertEqual(
            unicode(self.ui.ui.name_assistance.text()),
            setup_account.bad % setup_account.EMPTY_NAME
        )
        self.ui.hide()

    def test_valid_name(self):
        """Status when the name field is valid:

        * Name Assistance label invisible
        """
        self.ui.ui.name_edit.setText("John Doe")
        self.ui.name_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.name_assistance.isVisible(), False)
        self.ui.hide()

    def test_invalid_email(self):
        """Status when the email field has no @:

        * Email Assistance label visible
        * Correct error message in it
        """
        self.ui.ui.email_edit.setText("foobar")
        self.ui.email_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.email_assistance.isVisible(), True)
        self.assertEqual(
            unicode(self.ui.ui.email_assistance.text()),
            setup_account.bad % setup_account.INVALID_EMAIL
        )
        self.ui.hide()

    def test_valid_email(self):
        """Status when the email field has a @:

        * Email Assistance label invisible
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.email_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.email_assistance.isVisible(), False)
        self.ui.hide()

    def test_matching_emails(self):
        """Status when the email fields match:

        * Email Assistance label invisible
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.ui.confirm_email_edit.setText("foo@bar")
        self.ui.confirm_email_assistance()
        self.ui.show()
        self.assertEqual(
            self.ui.ui.confirm_email_assistance.isVisible(), False)
        self.ui.hide()

    def test_not_matching_emails(self):
        """Status when the email fields don't match:

        * Email Assistance label visible
        * Correct error message
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.ui.confirm_email_edit.setText("foo@baz")
        self.ui.confirm_email_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.confirm_email_assistance.isVisible(), True)
        self.assertEqual(
            unicode(self.ui.ui.confirm_email_assistance.text()),
            setup_account.bad % setup_account.EMAIL_MATCH
        )
        self.ui.hide()

    def test_short_password(self):
        """Status with short password:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("foobar")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.bad % setup_account.PASSWORD_LENGTH in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_long_password(self):
        """Status with long password:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("foobarbaz")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.good % setup_account.PASSWORD_LENGTH in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_no_number_password(self):
        """Status with password without a number:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("foobarbaz")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.bad % setup_account.PASSWORD_DIGIT in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_number_password(self):
        """Status with password with a number:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("foobarba7")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.good % setup_account.PASSWORD_DIGIT in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_no_uppercase_password(self):
        """Status with password without uppercase letters:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("foobarbaz")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.bad % setup_account.PASSWORD_UPPER in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_upper_password(self):
        """Status with password with uppercase letters:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("Foobarba7")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.good % setup_account.PASSWORD_UPPER in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_matching_passwords(self):
        """Status when the passwords match:

        * Password assistance doesn't contain the message
        """
        self.ui.ui.password_edit.setText("Foobarba7")
        self.ui.ui.confirm_password_edit.setText("Foobarba7")
        self.ui.password_assistance()
        self.assertEqual(
            False,
            setup_account.PASSWORD_MATCH in
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_not_matching_passwords(self):
        """Status when the passwords not match:

        * Password assistance contains the right message
        """
        self.ui.ui.password_edit.setText("Foobarba7")
        self.ui.ui.confirm_password_edit.setText("BazBarFo0")
        self.ui.password_assistance()
        self.assertEqual(
            True,
            setup_account.bad % setup_account.PASSWORD_MATCH in
            unicode(self.ui.ui.password_assistance.text()),
        )
